/*
 * File:		mdvdp.c
 *
 * Purpose:		Sample Plug-In for ATI Remote Wonder. Used to control
 *				Micro DVD Player.
 *
 * Author:		ATI
 *
 * Copyright:	Copyright (c) 2002 ATI Technologies Inc
 *
 * Use of this Software Development Kit is subject to the terms and conditions
 * of the ATI Technologies Inc. Software Development Kit License Agreement for 
 * the "REMOTE WONDER Plug-In SDK". If you have not accepted and agreed to this 
 * License, you have no rights to use the software contained herein.  
 */

#include "ammo.h"				// Required for ATI Remote Wonder plug-in SDK API

/*
 * Function:	WhatKeysDoYouWant
 *
 * Purpose:		Determines what groups of keys should be re-directed to
 *				the plug-in.
 *
 * Inputs:		None
 *
 * Returns:		A bit mask which indicates the groups of keys that should
 *				be re-directed to this plug-in.
 *
 * When called: This function is called once when the plug-in is first loaded,
 *				and again after calling Configure().
 *
 * Author:		ATI
 *
 * Copyright:	Copyright (c) 2002 ATI Technologies Inc
 */
DWORD WhatKeysDoYouWant (void)
{
	// For winamp we need to control the following keys:
	//
	// CUSTOM_MAPPED - provide list of programmable options
	// VOLUME_GROUP - adjust the playback volume
	// PLAY_GROUP - play, pause, stop, FF, RW
	// SETUP - toggle the preferences dialog
	// MENU - toggle the playlist editor
	// RESIZE - toggle doublesize mode
	// STOPWATCH - add current track as a bookmark

	return (VOLUME_GROUP|PLAY_GROUP|SETUP|MENU|ENTER|RESIZE|STOPWATCH|CURSOR_GROUP);
}


/*
 * Function:	EnumerateProgrammableFunction
 *
 * Purpose:		Enumerate the descriptions of the programamble
 *				functions provided by this plug-in.
 *
 * Inputs:		wIndex - index of requested description
 *
 * Returns:		Pointer to the programmable function description
 *				or NULL if wIndex is greater than the number of
 *				programmable functions provided by this plug-in.
 *
 * When called:	This function is called after any call to WhatKeysDoYouWant()
 *				that returns a request for custom mapped keys (CUSTOM_MAPPED).
 *
 * Author:		ATI
 *
 * Copyright:	Copyright (c) 2002 ATI Technologies Inc
 */
char *EnumerateProgrammableFunction (WORD wIndex)
{	
	// none for now ;)
	return NULL;
}


/*
 * Function:	Configure
 *
 * Purpose:		Displays any configuration/setup page required by
 *				this plug-in. 
 *
 * Inputs:		hWnd - handle to the parent window. The parent window
 *				is used to ensure correct modal behaviour when creating 
 *				dialog and message boxes in this plug-in.
 *
 * Returns:		None
 *
 * When called: This function is called when the "Configure" button in the
 *				ATI Remote Wonder software is pressed.
 *
 * Author:		ATI
 *
 * Copyright:	Copyright (c) 2002 ATI Technologies Inc
 */
void Configure (HANDLE hWnd)
{
	// Display an 'About' message box
	MessageBox(hWnd, "Micro DVD Player Plugin 1.0\n"
					 "Copyright (c) 2003, Safai Ma <tfma@hotmail.com>\n"
					 "\n"
					 "Play - start playback\n"
					 "Pause - pause playback\n"
					 "Stop - stop playback\n"
					 "Next - go to next chapter\n"
					 "Previous - go to previous chapter\n"
					 "\n"
					 "Menu - show specials dialog\n"
					 "Setup - show options dialog\n"
					 "Stopwatch - change time display\n"
					 "\n"
					 "Up/Down/Left/Right/OK arrow - menu navigation\n",
					 "Enter - select menu"
					 "About", MB_OK);
}


/*
 * Function:	AreYouInFocus
 *
 * Purpose:		Determines if this plug-in is currently in-focus.
 *
 * Inputs:		None
 *
 * Returns:		TRUE if the plug-in is currently in focus, FALSE
 *				otherwise.
 *
 * When called:	This function is called when a key event occurs that
 *				the plug-in has requested.
 *
 * Author:		ATI
 *
 * Copyright:	Copyright (c) 2002 ATI Technologies Inc
 */
int AreYouInFocus (void)
{
	// The Micro DVD Player main window uses a class name of 'MediaPlayerClassic'.
	
	HWND hMDVDP = FindWindow("TfrmControls", "Micro DVD Player Control Bar");
	HWND hForeground = GetForegroundWindow();

	DWORD dwProcessId = 0, dwProcessIdForeground = 1;
	GetWindowThreadProcessId(hMDVDP, &dwProcessId);
	GetWindowThreadProcessId(hForeground, &dwProcessIdForeground);

	if (hMDVDP == NULL)
	{
		// Micro DVD Player is not running
		return FALSE;
	}

	if (hMDVDP == hForeground)
	{
		// Micro DVD Player main window is in focus
		return TRUE;
	}
	
	if (hMDVDP == GetWindow(hForeground, GW_OWNER))
	{
		return TRUE;
	}

	if (dwProcessId == dwProcessIdForeground)
	{
		return TRUE;
	}

	// Micro DVD Player is running but is not in focus
	return FALSE;
}


/*
 * Function:	HandleKey
 *
 * Purpose:		Handles remote control keys.
 *
 * Inputs:		bCustom   - TRUE if wKeyEvent has been mapped to a custom
 *						    programamble option.
 *				wKeyEvent - Remote control key code for button that was
 *							pressed.
 *				wState    - RMCTRL_KEY_ON, when button is first pressed
 *						    RMCTRL_KEY_REPEAT, when button is held down
 *						    RMCTRL_KEY_OFF, when button is released
 *
 * Returns:		TRUE if the plug-in handled the command, FALSE otherwise.
 *
 * When called: This function is called whenever a key event occurs that
 *				the plug-in has requested when the plug-in is in focus.
 *
 * Author:		ATI
 *
 * Copyright:	Copyright (c) 2002 ATI Technologies Inc
 */
BOOL HandleKey (BOOL bCustom, WORD wKeyEvent, WORD wState)
{
	// Get the Micro DVD Player main window.
	HWND hMDVDP = FindWindow("TfrmControls", "Micro DVD Player Control Bar");

	// If the main window is not found Micro DVD Player has been closed.
	if (hMDVDP == NULL)
	{
		return FALSE;
	}
	
	// Handle programmed keys seperately
	if (bCustom)	
	{
		return FALSE;
	}

	// Handle non-programmed keys
	if (wState == RMCTRL_KEY_ON)
	{
		switch (wKeyEvent)
		{
		case RMCTRL_STOPWATCH:
			PostMessage(hMDVDP, WM_KEYUP, 'T', 0);
			return TRUE;

		case RMCTRL_RESIZE:
			PostMessage(hMDVDP, WM_KEYUP, 'Y', 0);
			return TRUE;

		case RMCTRL_MENU:
			PostMessage(hMDVDP, WM_KEYUP, 'S' /* show special feature list*/, 0);
			return TRUE;

		case RMCTRL_SETUP:
			PostMessage(hMDVDP, WM_KEYUP, VK_F12, 0);
			return TRUE;

		case RMCTRL_VOLUMEUP:
			PostMessage(hMDVDP, WM_KEYUP, VK_ADD, 0);
			return TRUE;
	
		case RMCTRL_VOLUMEDOWN:
			PostMessage(hMDVDP, WM_KEYUP, VK_SUBTRACT, 0);
			return TRUE;	
	
		case RMCTRL_MUTE:
			PostMessage(hMDVDP, WM_KEYUP, VK_MULTIPLY, 0);
			return TRUE;

		case RMCTRL_UP:
			PostMessage(hMDVDP, WM_KEYUP, VK_UP, 0);
			return TRUE;	

		case RMCTRL_DOWN:
			PostMessage(hMDVDP, WM_KEYUP, VK_DOWN, 0);
			return TRUE;	

		case RMCTRL_LEFT:
			PostMessage(hMDVDP, WM_KEYUP, VK_LEFT, 0);
			return TRUE;	

		case RMCTRL_RIGHT:
			PostMessage(hMDVDP, WM_KEYUP, VK_RIGHT, 0);
			return TRUE;	

		case RMCTRL_ENTER:
			PostMessage(hMDVDP, WM_KEYUP, VK_CONTROL, 0);
			return TRUE;	

		case RMCTRL_PLAY:
			PostMessage(hMDVDP, WM_KEYUP, VK_SPACE, 0);
			return TRUE;	

		case RMCTRL_PAUSE:
			PostMessage(hMDVDP, WM_KEYUP, VK_PAUSE, 0);
			return TRUE;	

		case RMCTRL_STOP:
			PostMessage(hMDVDP, WM_KEYUP, VK_BACK, 0);
			return TRUE;

		case RMCTRL_RW:
			PostMessage(hMDVDP, WM_KEYUP, 'P' /* prev file */, 0);
			return TRUE;

		case RMCTRL_FF:
			PostMessage(hMDVDP, WM_KEYUP, 'N' /* next file */, 0);
			return TRUE;
		}
	}
	else if (wState == RMCTRL_KEY_REPEAT)	
	{
		switch (wKeyEvent)
		{
		case RMCTRL_VOLUMEUP:
			PostMessage(hMDVDP, WM_KEYUP, VK_ADD, 0);
			return TRUE;
	
		case RMCTRL_VOLUMEDOWN:
			PostMessage(hMDVDP, WM_KEYUP, VK_SUBTRACT, 0);
			return TRUE;	
		}			
	}

	// The plug-in did not process this key. It is important to return FALSE
	// if the key was not processed so that other plug-ins or the ATI Remote Wonder
	// software can handle the key press.
	return FALSE;
}
